Attribute VB_Name = "bas_Draw_Moon_s4p"
Option Compare Database
Option Explicit

' module name: bas_Draw_Moon_s4p
'*************** Code Start ***************************************************
' Purpose  : draw a Moon on an Access report in any phase
'             specify report object, center coordinate and radius
'             optionally set colors, fraction lit,
'             and if moon is waxing or waning
'           USES Circle
' Author   : crystal (strive4peace)
' Code List: www.msaccessgurus.com/code.htm
' This code: https://msaccessgurus.com/VBA/Draw_Moon.htm
' LICENSE  :
'   You may freely use and share this code, but not sell it.
'   Keep attribution. Mark your changes. Use at your own risk.
'------------------------------------------------------------------------------
'           Global variables
'------------------------------------------------------------------------------
Public Const PI As Double = 3.14159
Public Const gZero As Double = 0.0000001
Public Const TWIPperINCH As Long = 1440

Public Const gColorGrayVeryLight As Long = 16448250 'RGB(250, 250, 250)
Public Const gColorGray As Long = 11513775 'RGB(200, 200, 200)
Public Const gColorMidnightBlue As Long = 7346457 'RGB(25, 25, 112)
Public Const gColorPaleYellow As Long = 9298389 'RGB(213, 225, 141)

Public Const gColorCyan As Long = 16769385 'RGB(105, 225, 255)

'------------------------------------------------------------------------------
'           Draw_Moon_s4p
' send center coordinate and size
'------------------------------------------------------------------------------
Public Sub Draw_Moon_s4p(poReport As Report _
      , pXCenter As Double _
      , pYCenter As Double _
      , ByVal pRadius As Double _
   , Optional pFractionLit As Single = 1 _
   , Optional pbWax As Boolean _
   , Optional pnColorLight As Long = vbWhite _
   , Optional pnColorDark As Long = vbBlack _
   , Optional pnColorOutline As Long = gColorGray _
   )
'230209, 11
   On Error GoTo Proc_Err
   
   'PARAMETERs
   '  poReport = report object
   '  pXCenter, pYCenter = center of moon
   '  pRadius = radius of moon
   '  pFractionLit = fraction that is lit, 0 to 1
   '  pbWax = True if waxing (light on right)
   '          False if waning (light on left)
   '  pnColorLight = color for the lit part of moon
   '  pnColorDark = color for the dark part of moon
   '  pnColorOutline = outline color, negative is no outline
   
   Dim nLeftColor As Long _
      , nRightColor As Long _
      , nMiddleColor As Long _
      , dbAngle1 As Double _
      , dbAngle2 As Double _
      , sgAspect As Single
    
   If pbWax = True Then 'light on the right
      nRightColor = pnColorLight
      nLeftColor = pnColorDark
   Else
      nRightColor = pnColorDark
      nLeftColor = pnColorLight
   End If
   If Abs(pFractionLit - 0.5) < 0.001 Then
      'no middle oval
   ElseIf pFractionLit > 0.5 Then
      nMiddleColor = pnColorLight
      sgAspect = 1 / ((pFractionLit - 0.5) * 2)
   Else
      nMiddleColor = pnColorDark
      sgAspect = 1 / ((0.5 - pFractionLit) * 2)
   End If
   
   With poReport
      
      .ScaleMode = 1 'twips

      .DrawWidth = 1 'iDrawWidth
      .FillStyle = 0 'Opaque
      
      
      If pFractionLit > 0.99999 Then 'full mooon
         .FillColor = pnColorLight
         poReport.Circle (pXCenter, pYCenter) _
               , pRadius _
               , pnColorLight
      ElseIf Abs(pFractionLit) < 0.0001 Then 'new moon
         .FillColor = pnColorDark
         poReport.Circle (pXCenter, pYCenter) _
               , pRadius _
               , pnColorDark
      Else
         'draw a filled half circle on the right
         dbAngle1 = gZero
         dbAngle2 = PI / 2
         .FillColor = nRightColor
         poReport.Circle (pXCenter, pYCenter) _
               , pRadius _
               , nRightColor _
               , -dbAngle1, -dbAngle2
         dbAngle1 = PI * 3 / 2
         dbAngle2 = PI
         poReport.Circle (pXCenter, pYCenter) _
               , pRadius _
               , nRightColor _
               , -dbAngle1, -dbAngle2
         'draw a filled half circle on the left
         dbAngle1 = PI / 2
         dbAngle2 = PI * 3 / 2
         .FillColor = nLeftColor
         poReport.Circle (pXCenter, pYCenter) _
               , pRadius _
               , nLeftColor _
               , -dbAngle1, -dbAngle2
         'draw middle oval
         If Abs(pFractionLit - 0.5) > 0.001 Then
            'draw oval to cover middle
            .FillColor = nMiddleColor
            poReport.Circle (pXCenter, pYCenter) _
                  , pRadius _
                  , nMiddleColor _
                  , , , sgAspect
         End If
      End If
      'draw outline
      If pnColorOutline >= 0 Then
         .FillStyle = 1 'transparent
         poReport.Circle (pXCenter, pYCenter) _
               , pRadius _
               , pnColorOutline
      End If
   End With 'poReport
   
Proc_Exit:
   On Error GoTo 0
   Exit Sub
  
Proc_Err:
   MsgBox Err.Description _
       , , "ERROR " & Err.Number _
        & "   Draw_Moon_s4p"

   Resume Proc_Exit
   Resume
End Sub

'*************** Code End *****************************************************

